@@ -40,10 +40,6 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#include "egg-discrete.h"
-#include "egg-debug.h"
-#include "egg-string.h"
-
 #include "gpm-brightness.h"
 #include "gpm-common.h"
 #include "gpm-marshal.h"
@@ -88,6 +84,41 @@ G_DEFINE_TYPE (GpmBrightness, gpm_brightness, G_TYPE_OBJECT)
 static guint signals [LAST_SIGNAL] = { 0 };
 static gpointer gpm_brightness_object = NULL;
 
+/**
+ * gpm_brightness_helper_strtoint:
+ * @text: The text the convert
+ * @value: The return numeric return value
+ *
+ * Converts a string into a signed integer value in a safe way.
+ *
+ * Return value: %TRUE if the string was converted correctly
+ **/
+static gboolean
+gpm_brightness_helper_strtoint (const gchar *text, gint *value)
+{
+	gchar *endptr = NULL;
+	gint64 value_raw;
+
+	/* invalid */
+	if (text == NULL)
+		return FALSE;
+
+	/* parse */
+	value_raw = g_ascii_strtoll (text, &endptr, 10);
+
+	/* parsing error */
+	if (endptr == text)
+		return FALSE;
+
+	/* out of range */
+	if (value_raw > G_MAXINT || value_raw < G_MININT)
+		return FALSE;
+
+	/* cast back down to value */
+	*value = (gint) value_raw;
+	return TRUE;
+}
+
 /**
  * gpm_brightness_helper_get_value:
  **/
@@ -106,14 +137,14 @@ gpm_brightness_helper_get_value (const gchar *argument)
 	ret = g_spawn_command_line_sync (command,
 					 &stdout_data, NULL, &exit_status, &error);
 	if (!ret) {
-		egg_error ("failed to get value: %s", error->message);
+		g_error ("failed to get value: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-	egg_debug ("executing %s retval: %i", command, exit_status);
+	g_debug ("executing %s retval: %i", command, exit_status);
 
 	/* parse for a number */
-	ret = egg_strtoint (stdout_data, &value);
+	ret = gpm_brightness_helper_strtoint (stdout_data, &value);
 	if (!ret)
 		goto out;
 out:
@@ -137,11 +168,11 @@ gpm_brightness_helper_set_value (const gchar *argument, gint value)
 	command = g_strdup_printf ("pkexec " SBINDIR "/gnome-power-backlight-helper --%s %i", argument, value);
 	ret = g_spawn_command_line_sync (command, NULL, NULL, &exit_status, &error);
 	if (!ret) {
-		egg_error ("failed to get value: %s", error->message);
+		g_error ("failed to get value: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-	egg_debug ("executing %s retval: %i", command, exit_status);
+	g_debug ("executing %s retval: %i", command, exit_status);
 out:
 	g_free (command);
 	return ret;
@@ -176,11 +207,14 @@ gpm_brightness_output_get_internal (GpmBrightness *brightness, RROutput output,
 
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
+	if (brightness->priv->backlight == None)
+		return FALSE;
+
 	if (XRRGetOutputProperty (brightness->priv->dpy, output, brightness->priv->backlight,
 				  0, 4, False, False, None,
 				  &actual_type, &actual_format,
 				  &nitems, &bytes_after, ((unsigned char **)&prop)) != Success) {
-		egg_debug ("failed to get property");
+		g_debug ("failed to get property");
 		return FALSE;
 	}
 	if (actual_type == XA_INTEGER && nitems == 1 && actual_format == 32) {
@@ -208,7 +242,7 @@ gpm_brightness_output_set_internal (GpmBrightness *brightness, RROutput output,
 	XFlush (brightness->priv->dpy);
 	gdk_flush ();
 	if (gdk_error_trap_pop ()) {
-		egg_warning ("failed to XRRChangeOutputProperty for brightness %i", value);
+		g_warning ("failed to XRRChangeOutputProperty for brightness %i", value);
 		ret = FALSE;
 	}
 	/* we changed the hardware */
@@ -228,24 +262,24 @@ gpm_brightness_setup_display (GpmBrightness *brightness)
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
 	/* get the display */
-	brightness->priv->dpy = GDK_DISPLAY();
+	brightness->priv->dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default());
 	if (!brightness->priv->dpy) {
-		egg_error ("Cannot open display");
+		g_error ("Cannot open display");
 		return FALSE;
 	}
 	/* is XRandR new enough? */
 	if (!XRRQueryVersion (brightness->priv->dpy, &major, &minor)) {
-		egg_debug ("RandR extension missing");
+		g_debug ("RandR extension missing");
 		return FALSE;
 	}
 	if (major < 1 || (major == 1 && minor < 2)) {
-		egg_debug ("RandR version %d.%d too old", major, minor);
+		g_debug ("RandR version %d.%d too old", major, minor);
 		return FALSE;
 	}
 	/* can we support BACKLIGHT */
 	brightness->priv->backlight = XInternAtom (brightness->priv->dpy, "BACKLIGHT", True);
 	if (brightness->priv->backlight == None) {
-		egg_debug ("No outputs have backlight property");
+		g_debug ("No outputs have backlight property");
 		return FALSE;
 	}
 	return TRUE;
@@ -263,16 +297,16 @@ gpm_brightness_setup_version (GpmBrightness *brightness)
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
 	/* get the display */
-	brightness->priv->dpy = GDK_DISPLAY();
+	brightness->priv->dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default());
 	if (!brightness->priv->dpy) {
-		egg_error ("Cannot open display");
+		g_error ("Cannot open display");
 		return FALSE;
 	}
 	if (!XRRQueryVersion (brightness->priv->dpy, &major, &minor)) {
 		return FALSE;
 	}
 	if (major == 1 && minor < 3) {
-		egg_debug ("RandR version %d.%d does not support XRRGetScreenResourcesCurrent", major, minor);
+		g_debug ("RandR version %d.%d does not support XRRGetScreenResourcesCurrent", major, minor);
 		return FALSE;
 	}
 	return TRUE;
@@ -293,11 +327,11 @@ gpm_brightness_output_get_limits (GpmBrightness *brightness, RROutput output,
 
 	info = XRRQueryOutputProperty (brightness->priv->dpy, output, brightness->priv->backlight);
 	if (info == NULL) {
-		egg_debug ("could not get output property");
+		g_debug ("could not get output property");
 		return FALSE;
 	}
 	if (!info->range || info->num_values != 2) {
-		egg_debug ("was not range");
+		g_debug ("was not range");
 		ret = FALSE;
 		goto out;
 	}
@@ -327,9 +361,9 @@ gpm_brightness_output_get_percentage (GpmBrightness *brightness, RROutput output
 	ret = gpm_brightness_output_get_limits (brightness, output, &min, &max);
 	if (!ret || min == max)
 		return FALSE;
-	egg_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
-	percentage = egg_discrete_to_percent (cur, (max-min)+1);
-	egg_debug ("percentage %i", percentage);
+	g_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
+	percentage = gpm_discrete_to_percent (cur, (max-min)+1);
+	g_debug ("percentage %i", percentage);
 	brightness->priv->shared_value = percentage;
 	return TRUE;
 }
@@ -353,14 +387,14 @@ gpm_brightness_output_down (GpmBrightness *brightness, RROutput output)
 	ret = gpm_brightness_output_get_limits (brightness, output, &min, &max);
 	if (!ret || min == max)
 		return FALSE;
-	egg_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
+	g_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
 	if (cur == min) {
-		egg_debug ("already min");
+		g_debug ("already min");
 		return TRUE;
 	}
 	step = gpm_brightness_get_step ((max-min)+1);
 	if (cur < step) {
-		egg_debug ("truncating to %i", min);
+		g_debug ("truncating to %i", min);
 		cur = min;
 	} else {
 		cur -= step;
@@ -387,14 +421,14 @@ gpm_brightness_output_up (GpmBrightness *brightness, RROutput output)
 	ret = gpm_brightness_output_get_limits (brightness, output, &min, &max);
 	if (!ret || min == max)
 		return FALSE;
-	egg_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
+	g_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
 	if (cur == max) {
-		egg_debug ("already max");
+		g_debug ("already max");
 		return TRUE;
 	}
 	cur += gpm_brightness_get_step ((max-min)+1);
 	if (cur > max) {
-		egg_debug ("truncating to %i", max);
+		g_debug ("truncating to %i", max);
 		cur = max;
 	}
 	ret = gpm_brightness_output_set_internal (brightness, output, cur);
@@ -423,16 +457,16 @@ gpm_brightness_output_set (GpmBrightness *brightness, RROutput output)
 	if (!ret || min == max)
 		return FALSE;
 
-	shared_value_abs = egg_discrete_from_percent (brightness->priv->shared_value, (max-min)+1);
-	egg_debug ("percent=%i, absolute=%i", brightness->priv->shared_value, shared_value_abs);
+	shared_value_abs = gpm_discrete_from_percent (brightness->priv->shared_value, (max-min)+1);
+	g_debug ("percent=%i, absolute=%i", brightness->priv->shared_value, shared_value_abs);
 
-	egg_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
+	g_debug ("hard value=%i, min=%i, max=%i", cur, min, max);
 	if (shared_value_abs > (gint) max)
 		shared_value_abs = max;
 	if (shared_value_abs < (gint) min)
 		shared_value_abs = min;
 	if ((gint) cur == shared_value_abs) {
-		egg_debug ("already set %i", cur);
+		g_debug ("already set %i", cur);
 		return TRUE;
 	}
 
@@ -441,7 +475,7 @@ gpm_brightness_output_set (GpmBrightness *brightness, RROutput output)
 
 		/* some adaptors have a large number of steps */
 		step = gpm_brightness_get_step (shared_value_abs - cur);
-		egg_debug ("using step of %i", step);
+		g_debug ("using step of %i", step);
 
 		/* going up */
 		for (i=cur; i<=shared_value_abs; i+=step) {
@@ -455,7 +489,7 @@ gpm_brightness_output_set (GpmBrightness *brightness, RROutput output)
 
 		/* some adaptors have a large number of steps */
 		step = gpm_brightness_get_step (cur - shared_value_abs);
-		egg_debug ("using step of %i", step);
+		g_debug ("using step of %i", step);
 
 		/* going down */
 		for (i=cur; i>=shared_value_abs; i-=step) {
@@ -485,7 +519,7 @@ gpm_brightness_foreach_resource (GpmBrightness *brightness, GpmXRandROp op, XRRS
 	/* do for each output */
 	for (i=0; i<resources->noutput; i++) {
 		output = resources->outputs[i];
-		egg_debug ("resource %i of %i", i+1, resources->noutput);
+		g_debug ("resource %i of %i", i+1, resources->noutput);
 		if (op==ACTION_BACKLIGHT_GET) {
 			ret = gpm_brightness_output_get_percentage (brightness, output);
 		} else if (op==ACTION_BACKLIGHT_INC) {
@@ -496,7 +530,7 @@ gpm_brightness_foreach_resource (GpmBrightness *brightness, GpmXRandROp op, XRRS
 			ret = gpm_brightness_output_set (brightness, output);
 		} else {
 			ret = FALSE;
-			egg_warning ("op not known");
+			g_warning ("op not known");
 		}
 		if (ret) {
 			success_any = TRUE;
@@ -519,11 +553,15 @@ gpm_brightness_foreach_screen (GpmBrightness *brightness, GpmXRandROp op)
 
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
+	/* Return immediately if we can't use XRandR */
+	if (!brightness->priv->has_extension)
+		return FALSE;
+
 	/* do for each screen */
 	length = brightness->priv->resources->len;
 	for (i=0; i<length; i++) {
 		resource = (XRRScreenResources *) g_ptr_array_index (brightness->priv->resources, i);
-		egg_debug ("using resource %p", resource);
+		g_debug ("using resource %p", resource);
 		ret = gpm_brightness_foreach_resource (brightness, op, resource);
 		if (ret)
 			success_any = TRUE;
@@ -543,7 +581,7 @@ gpm_brightness_trust_cache (GpmBrightness *brightness)
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 	/* only return the cached value if the cache is trusted and we have change events */
 	if (brightness->priv->cache_trusted && brightness->priv->has_changed_events) {
-		egg_debug ("using cache for value %u (okay)", brightness->priv->cache_percentage);
+		g_debug ("using cache for value %u (okay)", brightness->priv->cache_percentage);
 		return TRUE;
 	}
 
@@ -551,7 +589,7 @@ gpm_brightness_trust_cache (GpmBrightness *brightness)
 	 * if we have multiple things setting policy on the workstation, e.g. fast user switching
 	 * or kpowersave, then this will be invalid -- this logic may be insane */
 	if (GPM_SOLE_SETTER_USE_CACHE && brightness->priv->cache_trusted) {
-		egg_warning ("using cache for value %u (probably okay)", brightness->priv->cache_percentage);
+		g_warning ("using cache for value %u (probably okay)", brightness->priv->cache_percentage);
 		return TRUE;
 	}
 	return FALSE;
@@ -575,7 +613,7 @@ gpm_brightness_set (GpmBrightness *brightness, guint percentage, gboolean *hw_ch
 	/* can we check the new value with the cache? */
 	trust_cache = gpm_brightness_trust_cache (brightness);
 	if (trust_cache && percentage == brightness->priv->cache_percentage) {
-		egg_debug ("not setting the same value %i", percentage);
+		g_debug ("not setting the same value %i", percentage);
 		return TRUE;
 	}
 
@@ -590,7 +628,7 @@ gpm_brightness_set (GpmBrightness *brightness, guint percentage, gboolean *hw_ch
 	if (!ret) {
 		if (brightness->priv->extension_levels < 0)
 			brightness->priv->extension_levels = gpm_brightness_helper_get_value ("get-max-brightness");
-		brightness->priv->extension_current = egg_discrete_from_percent (percentage, brightness->priv->extension_levels+1);
+		brightness->priv->extension_current = gpm_discrete_from_percent (percentage, brightness->priv->extension_levels+1);
 		ret = gpm_brightness_helper_set_value ("set-brightness", brightness->priv->extension_current);
 	}
 
@@ -639,13 +677,13 @@ gpm_brightness_get (GpmBrightness *brightness, guint *percentage)
 		if (brightness->priv->extension_levels < 0)
 			brightness->priv->extension_levels = gpm_brightness_helper_get_value ("get-max-brightness");
 		brightness->priv->extension_current = gpm_brightness_helper_get_value ("get-brightness");
-		percentage_local = egg_discrete_to_percent (brightness->priv->extension_current, brightness->priv->extension_levels+1);
+		percentage_local = gpm_discrete_to_percent (brightness->priv->extension_current, brightness->priv->extension_levels+1);
 		ret = TRUE;
 	}
 
 	/* valid? */
 	if (percentage_local > 100) {
-		egg_warning ("percentage value of %i will be truncated", percentage_local);
+		g_warning ("percentage value of %i will be truncated", percentage_local);
 		percentage_local = 100;
 	}
 
@@ -672,6 +710,7 @@ gboolean
 gpm_brightness_up (GpmBrightness *brightness, gboolean *hw_changed)
 {
 	gboolean ret = FALSE;
+	guint step;
 
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
@@ -692,10 +731,21 @@ gpm_brightness_up (GpmBrightness *brightness, gboolean *hw_changed)
 		if (brightness->priv->extension_levels < 0)
 			brightness->priv->extension_levels = gpm_brightness_helper_get_value ("get-max-brightness");
 		brightness->priv->extension_current = gpm_brightness_helper_get_value ("get-brightness");
-		if (brightness->priv->extension_current < brightness->priv->extension_levels)
-			ret = gpm_brightness_helper_set_value ("set-brightness", ++brightness->priv->extension_current);
-		if (hw_changed != NULL)
-			*hw_changed = ret;
+
+		/* increase by the step, limiting to the maximum possible levels */
+		if (brightness->priv->extension_current < brightness->priv->extension_levels) {
+			step = gpm_brightness_get_step (brightness->priv->extension_levels);
+			brightness->priv->extension_current += step;
+			if (brightness->priv->extension_current > brightness->priv->extension_levels)
+				brightness->priv->extension_current = brightness->priv->extension_levels;
+			ret = gpm_brightness_helper_set_value ("set-brightness", brightness->priv->extension_current);
+			if (hw_changed != NULL)
+				*hw_changed = ret;
+		} else {
+			ret = TRUE;
+			if (hw_changed != NULL)
+				*hw_changed = FALSE;
+		}
 		brightness->priv->cache_trusted = FALSE;
 		goto out;
 	}
@@ -715,6 +765,7 @@ gboolean
 gpm_brightness_down (GpmBrightness *brightness, gboolean *hw_changed)
 {
 	gboolean ret = FALSE;
+	guint step;
 
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
@@ -732,11 +783,24 @@ gpm_brightness_down (GpmBrightness *brightness, gboolean *hw_changed)
 
 	/* legacy fallback */
 	if (!ret) {
+		if (brightness->priv->extension_levels < 0)
+			brightness->priv->extension_levels = gpm_brightness_helper_get_value ("get-max-brightness");
 		brightness->priv->extension_current = gpm_brightness_helper_get_value ("get-brightness");
-		if (brightness->priv->extension_current > 0)
-			ret = gpm_brightness_helper_set_value ("set-brightness", --brightness->priv->extension_current);
-		if (hw_changed != NULL)
-			*hw_changed = ret;
+
+		/* decrease by the step, limiting to zero */
+		if (brightness->priv->extension_current > 0) {
+			step = gpm_brightness_get_step (brightness->priv->extension_levels);
+			brightness->priv->extension_current -= step;
+			if (brightness->priv->extension_current < 0)
+				brightness->priv->extension_current = 0;
+			ret = gpm_brightness_helper_set_value ("set-brightness", brightness->priv->extension_current);
+			if (hw_changed != NULL)
+				*hw_changed = ret;
+		} else {
+			ret = TRUE;
+			if (hw_changed != NULL)
+				*hw_changed = FALSE;
+		}
 		brightness->priv->cache_trusted = FALSE;
 		goto out;
 	}
@@ -744,7 +808,6 @@ out:
 	return ret;
 }
 
-
 /**
  * gpm_brightness_may_have_changed:
  **/
@@ -755,10 +818,10 @@ gpm_brightness_may_have_changed (GpmBrightness *brightness)
 	guint percentage;
 	ret = gpm_brightness_get (brightness, &percentage);
 	if (!ret) {
-		egg_warning ("failed to get output");
+		g_warning ("failed to get output");
 		return;
 	}
-	egg_debug ("emitting brightness-changed (%i)", percentage);
+	g_debug ("emitting brightness-changed (%i)", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_CHANGED], 0, percentage);
 }
 
@@ -775,7 +838,6 @@ gpm_brightness_filter_xevents (GdkXEvent *xevent, GdkEvent *event, gpointer data
 	return GDK_FILTER_CONTINUE;
 }
 
-
 static void gpm_brightness_update_cache (GpmBrightness *brightness);
 
 /**
@@ -812,12 +874,12 @@ gpm_brightness_update_cache (GpmBrightness *brightness)
 	display = gdk_display_get_default ();
 	length = ScreenCount (brightness->priv->dpy);
 	for (screen = 0; screen < (gint) length; screen++) {
-		egg_debug ("screen %i of %i", screen+1, length);
+		g_debug ("screen %i of %i", screen+1, length);
 		gscreen = gdk_display_get_screen (display, screen);
 
 		/* if we have not setup the changed on the monitor, set it here */
 		if (g_object_get_data (G_OBJECT (gscreen), "gpk-set-monitors-changed") == NULL) {
-			egg_debug ("watching ::monitors_changed on %p", gscreen);
+			g_debug ("watching ::monitors_changed on %p", gscreen);
 			g_object_set_data (G_OBJECT (gscreen), "gpk-set-monitors-changed", (gpointer) "true");
 			g_signal_connect (G_OBJECT (gscreen), "monitors_changed",
 					  G_CALLBACK (gpm_brightness_monitors_changed), brightness);
@@ -837,7 +899,7 @@ gpm_brightness_update_cache (GpmBrightness *brightness)
 			resource = XRRGetScreenResources (brightness->priv->dpy, root);
 
 		if (resource != NULL) {
-			egg_debug ("adding resource %p", resource);
+			g_debug ("adding resource %p", resource);
 			g_ptr_array_add (brightness->priv->resources, resource);
 		}
 	}
@@ -925,15 +987,15 @@ gpm_brightness_init (GpmBrightness *brightness)
 	brightness->priv->has_randr13 = gpm_brightness_setup_version (brightness);
 #endif
 	if (brightness->priv->has_extension == FALSE)
-		egg_debug ("no XRANDR extension");
+		g_debug ("no XRANDR extension");
 
 	screen = gdk_screen_get_default ();
 	brightness->priv->root_window = gdk_screen_get_root_window (screen);
 	display = gdk_display_get_default ();
 
 	/* as we a filtering by a window, we have to add an event type */
-	if (!XRRQueryExtension (GDK_DISPLAY(), &event_base, &ignore)) {
-		egg_error ("can't get event_base for XRR");
+	if (!XRRQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default()), &event_base, &ignore)) {
+		g_warning ("can't get event_base for XRR");
 	}
 	gdk_x11_register_standard_event_type (display, event_base, RRNotify + 1);
 	gdk_window_add_filter (brightness->priv->root_window,
@@ -941,12 +1003,13 @@ gpm_brightness_init (GpmBrightness *brightness)
 
 	/* don't abort on error */
 	gdk_error_trap_push ();
-	XRRSelectInput (GDK_DISPLAY(), GDK_WINDOW_XID (brightness->priv->root_window),
+	XRRSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default()),
+			GDK_WINDOW_XID (brightness->priv->root_window),
 			RRScreenChangeNotifyMask |
 			RROutputPropertyNotifyMask); /* <--- the only one we need, but see rh:345551 */
 	gdk_flush ();
 	if (gdk_error_trap_pop ())
-		egg_warning ("failed to select XRRSelectInput");
+		g_warning ("failed to select XRRSelectInput");
 
 	/* create cache of XRRScreenResources as XRRGetScreenResources() is slow */
 	gpm_brightness_update_cache (brightness);
