@@ -22,9 +22,8 @@
 #include "config.h"
 
 #include <glib.h>
-#include <dbus/dbus-glib.h>
+#include <gio/gio.h>
 
-#include "egg-debug.h"
 #include "gpm-disks.h"
 
 static void     gpm_disks_finalize   (GObject	  *object);
@@ -33,7 +32,7 @@ static void     gpm_disks_finalize   (GObject	  *object);
 
 struct GpmDisksPrivate
 {
-	DBusGProxy		*proxy;
+	GDBusProxy		*proxy;
 	gchar			*cookie;
 };
 
@@ -48,29 +47,34 @@ static gboolean
 gpm_disks_unregister (GpmDisks *disks)
 {
 	gboolean ret = FALSE;
+	GVariant *retval = NULL;
 	GError *error = NULL;
 
-	/* no DeviceKit-disks */
+	/* no UDisks */
 	if (disks->priv->proxy == NULL) {
-		egg_warning ("no DeviceKit-disks");
+		g_warning ("no UDisks");
 		goto out;
 	}
 
-	/* clear spindown timeouts */
-	ret = dbus_g_proxy_call (disks->priv->proxy, "DriveUnsetAllSpindownTimeouts", &error,
-				 G_TYPE_STRING, disks->priv->cookie,
-				 G_TYPE_INVALID,
-				 G_TYPE_INVALID);
-	if (!ret) {
-		egg_warning ("failed to clear spindown timeout: %s", error->message);
+	/* set spindown timeouts */
+	retval = g_dbus_proxy_call_sync (disks->priv->proxy,
+					 "DriveUnsetAllSpindownTimeouts",
+					g_variant_new ("(s)",
+						       disks->priv->cookie),
+					G_DBUS_CALL_FLAGS_NONE,
+					-1, NULL, &error);
+	if (retval == NULL) {
+		/* abort as the DBUS method failed */
+		g_warning ("failed to clear spindown timeout: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-out:
-	/* reset */
-	g_free (disks->priv->cookie);
-	disks->priv->cookie = NULL;
 
+	/* sucess */
+	ret = TRUE;
+out:
+	if (retval != NULL)
+		g_variant_unref (retval);
 	return ret;
 }
 
@@ -81,28 +85,38 @@ static gboolean
 gpm_disks_register (GpmDisks *disks, gint timeout)
 {
 	gboolean ret = FALSE;
+	GVariant *retval = NULL;
+	GVariantBuilder array;
 	GError *error = NULL;
-	const gchar **options = {NULL};
 
-	/* no DeviceKit-disks */
+	/* no UDisks */
 	if (disks->priv->proxy == NULL) {
-		egg_warning ("no DeviceKit-disks");
+		g_warning ("no UDisks");
 		goto out;
 	}
 
 	/* set spindown timeouts */
-	ret = dbus_g_proxy_call (disks->priv->proxy, "DriveSetAllSpindownTimeouts", &error,
-				 G_TYPE_INT, timeout,
-				 G_TYPE_STRV, options,
-				 G_TYPE_INVALID,
-				 G_TYPE_STRING, &disks->priv->cookie,
-				 G_TYPE_INVALID);
-	if (!ret) {
-		egg_warning ("failed to set spindown timeout: %s", error->message);
+	g_variant_builder_init (&array, G_VARIANT_TYPE ("as"));
+	retval = g_dbus_proxy_call_sync (disks->priv->proxy,
+					 "DriveSetAllSpindownTimeouts",
+					g_variant_new ("(ias)",
+						       timeout,
+						       &array),
+					G_DBUS_CALL_FLAGS_NONE,
+					-1, NULL, &error);
+	if (retval == NULL) {
+		/* abort as the DBUS method failed */
+		g_warning ("failed to set spindown timeout: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
+
+	/* sucess */
+	ret = TRUE;
+	g_variant_get (retval, "(s)", &disks->priv->cookie);
 out:
+	if (retval != NULL)
+		g_variant_unref (retval);
 	return ret;
 }
 
@@ -116,20 +130,20 @@ gpm_disks_set_spindown_timeout (GpmDisks *disks, gint timeout)
 
 	/* get rid of last request */
 	if (disks->priv->cookie != NULL) {
-		egg_debug ("unregistering %s", disks->priv->cookie);
+		g_debug ("unregistering %s", disks->priv->cookie);
 		gpm_disks_unregister (disks);
 	}
 
 	/* is not enabled? */
 	if (timeout == 0) {
-		egg_debug ("disk spindown disabled");
+		g_debug ("disk spindown disabled");
 		goto out;
 	}
 
 	/* register */
 	ret = gpm_disks_register (disks, timeout);
 	if (ret)
-		egg_debug ("registered %s (%i)", disks->priv->cookie, timeout);
+		g_debug ("registered %s (%i)", disks->priv->cookie, timeout);
 out:
 	return ret;
 }
@@ -141,20 +155,23 @@ static void
 gpm_disks_init (GpmDisks *disks)
 {
 	GError *error = NULL;
-	DBusGConnection *connection;
+	GDBusConnection *connection;
 
 	disks->priv = GPM_DISKS_GET_PRIVATE (disks);
 
-	disks->priv->cookie = NULL;
-
 	/* get proxy to interface */
-	connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, NULL);
-	disks->priv->proxy = dbus_g_proxy_new_for_name_owner (connection,
-							      "org.freedesktop.DeviceKit.Disks",
-							      "/org/freedesktop/DeviceKit/Disks",
-							      "org.freedesktop.DeviceKit.Disks", &error);
+	connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
+	disks->priv->proxy =
+		g_dbus_proxy_new_sync (connection,
+			G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+			G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+			NULL,
+			"org.freedesktop.UDisks",
+			"/org/freedesktop/UDisks",
+			"org.freedesktop.UDisks",
+			NULL, &error);
 	if (disks->priv->proxy == NULL) {
-		egg_warning ("DBUS error: %s", error->message);
+		g_warning ("failed to setup disks proxy: %s", error->message);
 		g_error_free (error);
 	}
 }
