@@ -22,6 +22,7 @@
 #define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
 #define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
 #define g_marshal_value_peek_object(v)   g_value_get_object (v)
+#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
 #else /* !G_ENABLE_DEBUG */
 /* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
  *          Do not access GValues directly in your code. Instead, use the
@@ -45,16 +46,17 @@
 #define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
 #define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
 #define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
+#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
 
 /* NONE:INT,BOXED (gpm-marshal.list:1) */
 void
 gpm_marshal_VOID__INT_BOXED (GClosure     *closure,
-                             GValue       *return_value,
+                             GValue       *return_value G_GNUC_UNUSED,
                              guint         n_param_values,
                              const GValue *param_values,
-                             gpointer      invocation_hint,
+                             gpointer      invocation_hint G_GNUC_UNUSED,
                              gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_BOXED) (gpointer     data1,
@@ -88,10 +90,10 @@ gpm_marshal_VOID__INT_BOXED (GClosure     *closure,
 /* NONE:STRING,STRING (gpm-marshal.list:2) */
 void
 gpm_marshal_VOID__STRING_STRING (GClosure     *closure,
-                                 GValue       *return_value,
+                                 GValue       *return_value G_GNUC_UNUSED,
                                  guint         n_param_values,
                                  const GValue *param_values,
-                                 gpointer      invocation_hint,
+                                 gpointer      invocation_hint G_GNUC_UNUSED,
                                  gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer     data1,
@@ -122,21 +124,21 @@ gpm_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT,STRING,STRING (gpm-marshal.list:3) */
+/* NONE:POINTER,STRING,STRING (gpm-marshal.list:3) */
 void
-gpm_marshal_VOID__UINT_STRING_STRING (GClosure     *closure,
-                                      GValue       *return_value,
-                                      guint         n_param_values,
-                                      const GValue *param_values,
-                                      gpointer      invocation_hint,
-                                      gpointer      marshal_data)
+gpm_marshal_VOID__POINTER_STRING_STRING (GClosure     *closure,
+                                         GValue       *return_value G_GNUC_UNUSED,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint G_GNUC_UNUSED,
+                                         gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__UINT_STRING_STRING) (gpointer     data1,
-                                                         guint        arg_1,
-                                                         gpointer     arg_2,
-                                                         gpointer     arg_3,
-                                                         gpointer     data2);
-  register GMarshalFunc_VOID__UINT_STRING_STRING callback;
+  typedef void (*GMarshalFunc_VOID__POINTER_STRING_STRING) (gpointer     data1,
+                                                            gpointer     arg_1,
+                                                            gpointer     arg_2,
+                                                            gpointer     arg_3,
+                                                            gpointer     data2);
+  register GMarshalFunc_VOID__POINTER_STRING_STRING callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -152,10 +154,10 @@ gpm_marshal_VOID__UINT_STRING_STRING (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__UINT_STRING_STRING) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__POINTER_STRING_STRING) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_pointer (param_values + 1),
             g_marshal_value_peek_string (param_values + 2),
             g_marshal_value_peek_string (param_values + 3),
             data2);
@@ -164,10 +166,10 @@ gpm_marshal_VOID__UINT_STRING_STRING (GClosure     *closure,
 /* NONE:UINT,UINT,BOOL (gpm-marshal.list:4) */
 void
 gpm_marshal_VOID__UINT_UINT_BOOLEAN (GClosure     *closure,
-                                     GValue       *return_value,
+                                     GValue       *return_value G_GNUC_UNUSED,
                                      guint         n_param_values,
                                      const GValue *param_values,
-                                     gpointer      invocation_hint,
+                                     gpointer      invocation_hint G_GNUC_UNUSED,
                                      gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__UINT_UINT_BOOLEAN) (gpointer     data1,
@@ -203,10 +205,10 @@ gpm_marshal_VOID__UINT_UINT_BOOLEAN (GClosure     *closure,
 /* NONE:STRING,STRING,STRING (gpm-marshal.list:5) */
 void
 gpm_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
-                                        GValue       *return_value,
+                                        GValue       *return_value G_GNUC_UNUSED,
                                         guint         n_param_values,
                                         const GValue *param_values,
-                                        gpointer      invocation_hint,
+                                        gpointer      invocation_hint G_GNUC_UNUSED,
                                         gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING) (gpointer     data1,
@@ -242,10 +244,10 @@ gpm_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
 /* NONE:STRING,BOOLEAN (gpm-marshal.list:6) */
 void
 gpm_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
-                                  GValue       *return_value,
+                                  GValue       *return_value G_GNUC_UNUSED,
                                   guint         n_param_values,
                                   const GValue *param_values,
-                                  gpointer      invocation_hint,
+                                  gpointer      invocation_hint G_GNUC_UNUSED,
                                   gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_BOOLEAN) (gpointer     data1,
@@ -279,10 +281,10 @@ gpm_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
 /* NONE:STRING,STRING,BOOLEAN (gpm-marshal.list:7) */
 void
 gpm_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
-                                         GValue       *return_value,
+                                         GValue       *return_value G_GNUC_UNUSED,
                                          guint         n_param_values,
                                          const GValue *param_values,
-                                         gpointer      invocation_hint,
+                                         gpointer      invocation_hint G_GNUC_UNUSED,
                                          gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN) (gpointer     data1,
@@ -318,10 +320,10 @@ gpm_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
 /* NONE:STRING,STRING,BOOLEAN,BOOLEAN,BOOLEAN (gpm-marshal.list:8) */
 void
 gpm_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
-                                                         GValue       *return_value,
+                                                         GValue       *return_value G_GNUC_UNUSED,
                                                          guint         n_param_values,
                                                          const GValue *param_values,
-                                                         gpointer      invocation_hint,
+                                                         gpointer      invocation_hint G_GNUC_UNUSED,
                                                          gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN_BOOLEAN) (gpointer     data1,
@@ -365,10 +367,10 @@ gpm_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
 /* NONE:INT,LONG,BOOLEAN,BOOLEAN (gpm-marshal.list:11) */
 void
 gpm_marshal_VOID__INT_LONG_BOOLEAN_BOOLEAN (GClosure     *closure,
-                                            GValue       *return_value,
+                                            GValue       *return_value G_GNUC_UNUSED,
                                             guint         n_param_values,
                                             const GValue *param_values,
-                                            gpointer      invocation_hint,
+                                            gpointer      invocation_hint G_GNUC_UNUSED,
                                             gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_LONG_BOOLEAN_BOOLEAN) (gpointer     data1,
@@ -406,10 +408,10 @@ gpm_marshal_VOID__INT_LONG_BOOLEAN_BOOLEAN (GClosure     *closure,
 /* NONE:BOOLEAN,BOOLEAN (gpm-marshal.list:12) */
 void
 gpm_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
-                                   GValue       *return_value,
+                                   GValue       *return_value G_GNUC_UNUSED,
                                    guint         n_param_values,
                                    const GValue *param_values,
-                                   gpointer      invocation_hint,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__BOOLEAN_BOOLEAN) (gpointer     data1,
@@ -445,10 +447,10 @@ gpm_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
 /* NONE:UINT,UINT (gpm-marshal.list:14) */
 void
 gpm_marshal_VOID__UINT_UINT (GClosure     *closure,
-                             GValue       *return_value,
+                             GValue       *return_value G_GNUC_UNUSED,
                              guint         n_param_values,
                              const GValue *param_values,
-                             gpointer      invocation_hint,
+                             gpointer      invocation_hint G_GNUC_UNUSED,
                              gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__UINT_UINT) (gpointer     data1,
